package com.gic.enterprise.service.outer.impl;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.CollectionUtil;
import com.gic.enterprise.constant.billing.FeeTypeEnum;
import com.gic.enterprise.dto.HomeStatisticsDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.HomeStatisticsQO;
import com.gic.enterprise.service.HomeStatisticsApiService;
import com.gic.enterprise.utils.chart.FeeChartUtils;
import com.gic.enterprise.utils.valid.ValidParamsUtils;

@Service("homeStatisticsApiService")
public class HomeStatisticsApiServiceImpl implements HomeStatisticsApiService {

    private static final Logger LOGGER = LoggerFactory.getLogger(HomeStatisticsApiServiceImpl.class);
    @Override
    public ServiceResponse<List<HomeStatisticsDTO>> viewFeeCharts(HomeStatisticsQO params) {
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(params, HomeStatisticsQO.SelectValidView.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        FeeTypeEnum feeTypeEnum = FeeTypeEnum.getEnumByCode(params.getFeeType());
        if (feeTypeEnum == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "消费类型有误");
        }
        //获取时间段数据，根据开始时间和结束时间
        List<String> dateList = null;
        try {
            dateList = getDateList(params.getStartTime(), params.getEndTime());
        } catch (Exception e) {
            e.printStackTrace();
            return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), e.getMessage());
        }
        if (feeTypeEnum == FeeTypeEnum.TOTAL) {
            //如果是总共消费，则需要把所有的消费逐一累加处理
            Map<String, Double> tempMap = new HashMap<>(16);
            for (String str : dateList) {
                tempMap.put(str, 0D);
            }
            for (FeeTypeEnum feeEnum : FeeTypeEnum.values()) {
                params.setFeeType(feeEnum.getCode());
                List<HomeStatisticsDTO> list = FeeChartUtils.getFeeChart(params);
                for (HomeStatisticsDTO tempData : list) {
                    //累加数据
                    String date = tempData.getStatisticsDate();
                    if (StringUtils.isBlank(date)) {
                        continue;
                    }
                    tempMap.put(tempData.getStatisticsDate(),
                            tempData.getFee() + tempMap.get(tempData.getStatisticsDate()));
                }
            }
            //map to list
            List<HomeStatisticsDTO> tempDTOList = new ArrayList<>(dateList.size());
            for (Map.Entry<String, Double> entry : tempMap.entrySet()) {
                HomeStatisticsDTO dto = new HomeStatisticsDTO();
                dto.setStatisticsDate(entry.getKey());
                dto.setFee(entry.getValue());
                tempDTOList.add(dto);
            }
            sort(tempDTOList);
            return ServiceResponse.success(tempDTOList);
        } else {
            List<HomeStatisticsDTO> list = FeeChartUtils.getFeeChart(params);
            dealWithHomeStatistics(list, dateList);
            return ServiceResponse.success(list);
        }
    }

    @Override
    public ServiceResponse<Map<String, Double>> getFee(Integer enterpriseId, Integer dateType) {
        Map<String, Double> feeMap = new HashMap<>(16);
        for (FeeTypeEnum feeTypeEnum : FeeTypeEnum.values()) {
            if (feeTypeEnum == FeeTypeEnum.TOTAL) {
                continue;
            } else {
                feeMap.put(feeTypeEnum.getMessage(),
                        FeeChartUtils.getFee(enterpriseId, dateType, feeTypeEnum.getCode()));
            }
        }
        return ServiceResponse.success(feeMap);
    }

    /**
     * 处理趋势数据，如果数据库中查询的数据缺少某一天的数据，则补齐改天0消费
     * @Title: dealWithHomeStatistics 
     * @Description:
      * @author guojuxing
     * @param list
     * @param dateList 
     * @return  java.util.List<com.gic.enterprise.dto.HomeStatisticsDTO> 
      */
    private static List<HomeStatisticsDTO> dealWithHomeStatistics(List<HomeStatisticsDTO> list, List<String> dateList) {
        Map<String, HomeStatisticsDTO> map = CollectionUtil.toMap(list, "statisticsDate");
        for (String date : dateList) {
            if (!map.containsKey(date)) {
                list.add(createHomeStatistics(date));
            }
        }
        //排序
        sort(list);
        return list;
    }

    private static void sort(List<HomeStatisticsDTO> list) {
        //排序
        Collections.sort(list, new Comparator<HomeStatisticsDTO>() {
            @Override
            public int compare(HomeStatisticsDTO o1, HomeStatisticsDTO o2) {
                return o1.getStatisticsDate().compareTo(o2.getStatisticsDate());
            }
        });
    }

    private static HomeStatisticsDTO createHomeStatistics(String date) {
        HomeStatisticsDTO dto = new HomeStatisticsDTO();
        dto.setFee(0D);
        dto.setStatisticsDate(date);
        return dto;
    }

    public static List<String> getDateList(String dateFirst, String dateSecond) throws ParseException {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date dateOne = dateFormat.parse(dateFirst);
        Date dateTwo = dateFormat.parse(dateSecond);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(dateOne);

        List<String> dateList = new ArrayList<>();
        while (calendar.getTime().before(dateTwo)) {
            dateList.add(dateFormat.format(calendar.getTime()));
            calendar.add(Calendar.DAY_OF_MONTH, 1);
        }
        dateList.add(dateSecond);
        return dateList;
    }

}
