package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AuditorDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.AuditorApiService;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.GlobalInfo;
import com.gic.enterprise.constant.AuditTypeEnum;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.constant.DataSecurityAuditEnum;
import com.gic.enterprise.dto.AuditLogDTO;
import com.gic.enterprise.dto.DictDTO;
import com.gic.enterprise.entity.TabRiskMode;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.AuditLogApiService;
import com.gic.enterprise.service.DictApiService;
import com.gic.enterprise.service.RiskModeApiService;
import com.gic.enterprise.service.RiskModeService;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.redisson.api.RBucket;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-10-29 10:21
 */
@Service("riskModeApiService")
public class RiskModeApiServiceImpl implements RiskModeApiService {

    private Logger logger = LogManager.getLogger(RiskModeApiServiceImpl.class);

    @Autowired
    private DictApiService dictApiService;
    @Autowired
    RiskModeService riskModeService;
    @Autowired
    private AuditLogApiService auditLogApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private AuditorApiService auditorApiService;

    @Override
    public ServiceResponse<Long> getRiskModeRemainingTime(Integer enterpriseId, Integer userId) {
        String redisKey = Constants.RISK_MODE_REDIS_KEY + enterpriseId + ":" + userId;
        RedissonClient redisClient = RedisUtil.getRedisClient();
        RBucket<Object> bucket = redisClient.getBucket(redisKey);
        if (bucket.get() != null) {
            long ttlTime = bucket.remainTimeToLive();
            return EnterpriseServiceResponse.success(ttlTime/1000);
        } else {
            return EnterpriseServiceResponse.success();
        }
    }

    @Override
    public ServiceResponse<Void> checkRiskMode(Integer enterpriseId, Integer userId) {
        String redisKey = Constants.RISK_MODE_REDIS_KEY + enterpriseId + ":" + userId;
        RedissonClient redisClient = RedisUtil.getRedisClient();
        RBucket<Object> bucket = redisClient.getBucket(redisKey);
        if (bucket.get() != null) {
            return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "当前已经是风险模式");
        }
        ServiceResponse<List<AuditorDTO>> auditorResponse = auditorApiService.listAuditorByProject(Constants.PROJECT_ITEM_RISK_MODE_APPLY, userId, enterpriseId);
        if (!auditorResponse.isSuccess()) {
            return EnterpriseServiceResponse.failure(auditorResponse.getCode(), auditorResponse.getMessage());
        }
        List<AuditorDTO> auditorList = auditorResponse.getResult();
        List<AuditorDTO> openidList = auditorList.stream().filter(e -> StringUtils.isNotBlank(e.getOpenid())).collect(Collectors.toList());
        List<AuditorDTO> userIdList = auditorList.stream().filter(e -> e.getUserId() != null).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(openidList) && CollectionUtils.isEmpty(userIdList)) {
            return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "审核员没有绑定微信或管理员");
        }
        TabRiskMode lastAudit = riskModeService.getLastWaitAuditByApplyUser(enterpriseId, userId);
        if (lastAudit != null) {
            long time = new Date().getTime() - lastAudit.getApplyTime().getTime();
            if (time < com.gic.enterprise.constant.Constants.FIVE_MINUTE) {
                // 5分钟内有未审核的申请
                return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "5分钟内有未审核的申请，请稍后再申请");
            }
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> auditCallback(String json) {
        logger.info("审批回调:{}", json);
        JSONObject jsonObject = JSON.parseObject(json);
        Integer auditResult = jsonObject.getInteger("auditResult");
        Integer riskModeId = jsonObject.getInteger("riskModeId");
        if (DataSecurityAuditEnum.PASS.getCode().equals(auditResult)) {
            TabRiskMode riskMode = riskModeService.getById(riskModeId);
            DictDTO dictDTO = dictApiService.getDict(Constants.RISK_MODE_DURATION_CATEGORY, riskMode.getDurationCode().toString()).getResult();
            String redisKey = Constants.RISK_MODE_REDIS_KEY + riskMode.getEnterpriseId() + ":" + riskMode.getApplyUserId();
            RedisUtil.setCache(redisKey, 1, Long.valueOf(dictDTO.getContent()), TimeUnit.MINUTES);
        }
        TabRiskMode riskMode = new TabRiskMode();
        riskMode.setRiskModeId(riskModeId);
        riskMode.setAuditResult(auditResult);
        this.riskModeService.update(riskMode);

        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> applyRiskMode(Integer enterpriseId, Integer userId, Integer duration, String applyReason) {
        if (duration == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
        ServiceResponse<DictDTO> serviceDict = dictApiService.getDict(Constants.RISK_MODE_DURATION_CATEGORY, duration.toString());
        if (!serviceDict.isSuccess() || serviceDict.getResult() == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }

        // 如果存在5分钟外未审核的申请， 把这个申请设为过期
        TabRiskMode lastAudit = riskModeService.getLastWaitAuditByApplyUser(enterpriseId, userId);
        if (lastAudit != null) {
            // 风险模式  判断5分钟内是否存在未审核的申请 有的话返回
            // 有空改成localDateTime
            long time = new Date().getTime() - lastAudit.getApplyTime().getTime();
            if (time < com.gic.enterprise.constant.Constants.FIVE_MINUTE) {
                // 5分钟内有未审核的申请
                return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "5分钟内有未审核的申请，请稍后再申请");
            }
            lastAudit.setStatus(GlobalInfo.DATA_STATUS_DELETE);
            riskModeService.update(lastAudit);
        }

        // 保存
        Date applyTime = new Date();
        Integer riskModeId = this.riskModeService.save(enterpriseId, userId, duration, applyTime, applyReason);
        UserDTO user = userApiService.getUserInfoById(userId).getResult();
        // 申请
        AuditLogDTO auditLogDTO = new AuditLogDTO();
        auditLogDTO.setEnterpriseId(enterpriseId);
        auditLogDTO.setApplyUserId(userId);
        auditLogDTO.setApplyUserName(user.getUserName());
        auditLogDTO.setApplyTime(applyTime);
        auditLogDTO.setProjectItemCode(Constants.PROJECT_ITEM_RISK_MODE_APPLY);
        Map<String, Object> map = new HashMap<>(4);
        map.put("riskModeId", riskModeId);
        auditLogDTO.setExtraInfo(JSON.toJSONString(map));
        auditLogDTO.setAuditType(AuditTypeEnum.RISK_MODE.getCode());
        ServiceResponse<Void> serviceResponse = auditLogApiService.apply(auditLogDTO);
        return serviceResponse;
    }
}